import json
import math
import argparse

# Claytor Lake: 37.076574 -80.590214
FIXED_LATITUDE = '37 13.937 0 N'
FIXED_LONGITUDE = '80 25.443 0 W'


def calculate_great_circle_distance(lat1, lon1, lat2, lon2):
    """Calculate the great-circle distance using coordinates.
    For example,
    lat = 50.06638888888889 or '50 03 59 N',
    lon = -5.714722222222222 or '005 42 53 W',
    negative indicates west/south.
    """
    # Convert deg-min-sec to decimal value
    def _convert_to_decimal_format(coordinate):
        if type(coordinate).__name__ == 'str':
            coordinate_list = coordinate.split(' ')
            deg = float(coordinate_list[0])
            minute = float(coordinate_list[1])
            sec = float(coordinate_list[2])
            dec = deg + minute/60.0 + sec/3600.0
            return dec if coordinate_list[3] == 'N' or coordinate_list[3] == 'E' else -dec
        elif type(coordinate).__name__ == 'float':
            return coordinate
        else:
            raise TypeError('Invalid format.')

    lat1 = _convert_to_decimal_format(lat1)
    lon1 = _convert_to_decimal_format(lon1)
    lat2 = _convert_to_decimal_format(lat2)
    lon2 = _convert_to_decimal_format(lon2)
    radius = 6371000  # Earth radius in meter.
    dlat = math.radians(lat2-lat1)
    dlon = math.radians(lon2-lon1)
    a = math.sin(dlat/2) * math.sin(dlat/2) + math.cos(math.radians(lat1)) \
        * math.cos(math.radians(lat2)) * math.sin(dlon/2) * math.sin(dlon/2)
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
    d = radius * c
    return d


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('sensor_input', help='sensor input in JSON format')
    args = parser.parse_args()
    try:
        sensor_data = json.loads(args.sensor_input)
    except Exception:
        print('')
    else:
        gps_data_list = sensor_data['GPS'].split(',')
        utc_time = gps_data_list[1]
        latitude = str(gps_data_list[3][0:2] + ' ' + gps_data_list[3][2:] + ' 0' + ' ' + gps_data_list[4])
        longitude = str(gps_data_list[5][0:3] + ' ' + gps_data_list[5][3:] + ' 0' + ' ' + gps_data_list[6])
        speed = float(gps_data_list[7]) * 0.514444  # Convert knot to m/s.
        distance = calculate_great_circle_distance(FIXED_LATITUDE, FIXED_LONGITUDE, latitude, longitude)
        print(' {0} {1} {2}'.format(distance, speed, utc_time))
